/** 
 * @file Graphics.h
 * @brief `NXwb_t@C
 * @note `Ɋւ鏈s<br>
 * @date 2008/04/17
 */

#ifndef _GRAPHICS_H_
#define _GRAPHICS_H_

#include <d3d9.h>
#include <d3dx9.h>
#include <tchar.h>
#include "AquaGameSystemTypedef.h"

namespace AquaGameSystem {

	//======================================
	// class tagGraphicsSpriteObject
	//======================================
	/** @brief XvCg`IuWFNg\
	 * @note 񎟌`pIuWFNg\<br>
	 */
	struct tagGraphicsSpriteObject {
		LPDIRECT3DTEXTURE9		m_pTexture;					/// eNX`
		float					m_fTextureWidth;				/// eNX`
		float					m_fTextureHeight;				/// eNX`
		TCHAR					m_szResFileName[MAX_PATH];		/// eNX`T[tF[XWJ摜t@C
		float					m_fResWidth;					/// eNX`T[tF[XWJ摜
		float					m_fResHeight;					/// eNX`T[tF[XWJ摜

		/// ftHgRXgN^
		tagGraphicsSpriteObject();
		/// fXgN^
		virtual ~tagGraphicsSpriteObject();
	};

	//======================================
	// class CGraphicsAbstract
	//======================================
	/** @brief `NX(ۃNX)
	 * @note GNX|[gpۃNX<br>
	 */
	class CGraphicsAbstract
	{
	public:
		virtual int							Init(HINSTANCE hInstance, HWND hWnd, UINT Width, UINT Height, BOOL bWndMode) = 0;
		virtual void						Term(void) = 0;
		virtual LPD3DXFONT					CreateFont(const D3DXFONT_DESC* pFont) = 0;
		virtual LPDIRECT3D9					GetD3D(void) const = 0;
		virtual LPDIRECT3DDEVICE9			GetD3DDevice(void) const = 0;
		virtual LPDIRECT3DSURFACE9			GetBackBuffer(void) const = 0;
		virtual HRESULT 					BeginScene(void) = 0;
		virtual HRESULT 					EndScene(void) = 0;
		virtual HRESULT						Clear(DWORD Count, CONST D3DRECT* pRects, DWORD Flags, D3DCOLOR Color, float Z, DWORD Stencil) = 0;
		virtual HRESULT						Present(CONST RECT* pSourceRect, CONST RECT* pDestRect, HWND hDestWindowOverride, CONST RGNDATA* pDirtyRegion) = 0;
		virtual void						FpsCntProc(void) = 0;
		virtual void						DispFps(void) = 0;
		virtual int							TextOut(LPD3DXFONT lpFont, UINT draw_x, UINT draw_y, LPCSTR pString, INT Count, D3DCOLOR Color) = 0;
		virtual int							TextOut(UINT draw_x, UINT draw_y, LPCSTR pString, INT Count, D3DCOLOR Color) = 0;
		virtual int							DrawSurface(LPDIRECT3DSURFACE9 pSourceSurface, CONST RECT *pSourceRect, UINT DestLeft, UINT DestTop) const = 0;
		virtual int							StretchRect(LPDIRECT3DSURFACE9 pSourceSurface, CONST RECT *pSourceRect, LPDIRECT3DSURFACE9 pDestSurface, CONST RECT *pDestRect, D3DTEXTUREFILTERTYPE Filter) = 0;
		virtual int							PrintScreen(IDirect3DSurface9 **ppSurface) = 0;
		virtual int							PrintScreen(HWND hWnd, LPCTSTR pDestFile) = 0;
		virtual tagGraphicsSpriteObject*	CreateSpriteObject(void) = 0;
		virtual BOOL						CreateSpriteTexture(tagGraphicsSpriteObject* pSpriteObject, UINT Width, UINT Height) = 0;
		virtual BOOL						CreateSpriteTexture(tagGraphicsSpriteObject* pSpriteObject, UINT Width, UINT Height, UINT Levels, DWORD Usage, D3DFORMAT Format, D3DPOOL Pool) = 0;
		virtual BOOL						LoadSpriteImageFromFile(tagGraphicsSpriteObject* pSpriteObject, LPCSTR pszFileName, const RECT* pDstRect, const RECT* pSrcRect) = 0;
		virtual BOOL						DrawSprite(tagGraphicsSpriteObject* pSpriteObject, VECTOR2_FLOAT vector2, float fR = 0.0f, D3DCOLOR AlphaColorRGBA = D3DCOLOR_RGBA(0xff, 0xff, 0xff, 0xff), BOOL bReverseU = FALSE, BOOL bReverseV = FALSE) = 0;
		virtual BOOL						DrawSprite(tagGraphicsSpriteObject* pSpriteObject, VECTOR2_FLOAT vector2, RECT_FLOAT TextureRect, float fR = 0.0f, D3DCOLOR AlphaColorRGBA = D3DCOLOR_RGBA(0xff, 0xff, 0xff, 0xff), BOOL bReverseU = FALSE, BOOL bReverseV = FALSE) = 0;
	};

	//======================================
	// class CGraphics
	//======================================
	/** @brief `NX
	 * @note `Ɋւ鏈s<br>
	 */
	class CGraphics : public CGraphicsAbstract
	{
	private:
		static CGraphics*				st_Instance;			/// CX^X
		LPDIRECT3D9						m_lpD3D;				/// DirectGraphicsIuWFNg
		LPDIRECT3DDEVICE9				m_lpD3DDevice;			/// DirectGraphicsfoCXIuWFNg
		LPDIRECT3DSURFACE9				m_lpBackBuffer;			/// obNobt@|C^
		D3DPRESENT_PARAMETERS			m_D3DPP;				/// v[e[Vp[^
		LPD3DXFONT						m_lpD3DXFont;			/// ftHgtHg
		int								m_iFpsLastUpdate;		/// FPSl
		int								m_iFpsCnt;				/// FPSlJE^
		DWORD							m_dwFpsTimer;			/// FPSv

		LPDIRECT3DVERTEXBUFFER9			m_pVTexBuff;			/// _obt@

		/// obNobt@
		int			InitBackBuffer(void);
		/// tHg
		int			InitFont(void);
		/// D3D[UIuWFNg
		int			InitD3DObject(void);
		/// D3D[UIuWFNg
		void		CleanupD3DObject(void);

	public:
		/// ftHgRXgN^
		CGraphics();
		/// fXgN^
		virtual ~CGraphics();
		/// CX^X̎擾
		static CGraphics*				GetInstance(void);
		/// ֐
		virtual int						Init(HINSTANCE hInstance, HWND hWnd, UINT Width, UINT Height, BOOL bWndMode);
		/// I֐
		virtual void					Term(void);
		/// tHg
		virtual LPD3DXFONT				CreateFont(const D3DXFONT_DESC* pFont);

		/// DirectGraphicsIuWFNg擾
		virtual LPDIRECT3D9				GetD3D(void) const;
		/// DirectGraphicsfoCXIuWFNg擾
		virtual LPDIRECT3DDEVICE9		GetD3DDevice(void) const;
		/// obNobt@|C^擾
		virtual LPDIRECT3DSURFACE9		GetBackBuffer(void) const;

		/// V[Jn
		virtual HRESULT 				BeginScene(void);
		/// BeginScene\bȟĂяoɂĊJnꂽV[I
		virtual HRESULT 				EndScene(void);
		/// r[|[g܂̓r[|[g̋`Zbgw肵 RGBA FɃNAA[xobt@NAAXeV obt@폜
		virtual HRESULT					Clear(DWORD Count, CONST D3DRECT* pRects, DWORD Flags, D3DCOLOR Color, float Z, DWORD Stencil);
		/// foCXLobN obt@̃V[PX̒ŁÃobt@̃Recv[e[V
		virtual HRESULT					Present(CONST RECT* pSourceRect, CONST RECT* pDestRect, HWND hDestWindowOverride, CONST RGNDATA* pDirtyRegion);
		/// FPSv
		virtual void					FpsCntProc(void);
		/// FPS\
		virtual void					DispFps(void);
		/// eLXg`itHgwj
		virtual int						TextOut(LPD3DXFONT lpFont, UINT draw_x, UINT draw_y, LPCSTR pString, INT Count, D3DCOLOR Color);
		/// eLXg`
		virtual int						TextOut(UINT draw_x, UINT draw_y, LPCSTR pString, INT Count, D3DCOLOR Color);
		/// T[tFCXobNobt@ɕ`悷
		virtual int						DrawSurface(LPDIRECT3DSURFACE9 pSourceSurface, CONST RECT *pSourceRect, UINT DestLeft, UINT DestTop) const;
		/// ]`̓e]`ɃRs[B
		virtual int						StretchRect(LPDIRECT3DSURFACE9 pSourceSurface, CONST RECT *pSourceRect, LPDIRECT3DSURFACE9 pDestSurface, CONST RECT *pDestRect, D3DTEXTUREFILTERTYPE Filter);
		/// ʂ̃XN[VbgIDirect3DSurface9ɕۑ
		virtual int						PrintScreen(IDirect3DSurface9 **ppSurface);
		/// ʂ̃XN[Vbgt@Cɕۑ
		virtual int						PrintScreen(HWND hWnd, LPCTSTR pDestFile);

		/// XvCg`IuWFNg𐶐
		virtual tagGraphicsSpriteObject*	CreateSpriteObject(void);
		/// XvCg`eNX`̍쐬
		virtual BOOL						CreateSpriteTexture(tagGraphicsSpriteObject* pSpriteObject, UINT Width, UINT Height);
		/// XvCg`eNX`̍쐬
		virtual BOOL						CreateSpriteTexture(tagGraphicsSpriteObject* pSpriteObject, UINT Width, UINT Height, UINT Levels, DWORD Usage, D3DFORMAT Format, D3DPOOL Pool);

		/// XvCg`eNX`ɃC[Wǂݍ
		virtual BOOL						LoadSpriteImageFromFile(tagGraphicsSpriteObject* pSpriteObject, LPCSTR pszFileName, const RECT* pDstRect, const RECT* pSrcRect);
		/// XvCg`eNX``悷
		virtual BOOL						DrawSprite(tagGraphicsSpriteObject* pSpriteObject, VECTOR2_FLOAT vector2, float fR = 0.0f, D3DCOLOR AlphaColorRGBA = D3DCOLOR_RGBA(0xff, 0xff, 0xff, 0xff), BOOL bReverseU = FALSE, BOOL bReverseV = FALSE);
		/// XvCg`eNX``悷
		virtual BOOL						DrawSprite(tagGraphicsSpriteObject* pSpriteObject, VECTOR2_FLOAT vector2, RECT_FLOAT TextureRect, float fR = 0.0f, D3DCOLOR AlphaColorRGBA = D3DCOLOR_RGBA(0xff, 0xff, 0xff, 0xff), BOOL bReverseU = FALSE, BOOL bReverseV = FALSE);



	};

}






#endif // _GRAPHICS_H_
