/** 
 * @file Input.h
 * @brief [U[C^[tFCXNXwb_t@C
 * @note [U[C^[tFCXɊւ鏈s<br>
 * @date 2008/04/17
 */

#ifndef _INPUT_H_
#define _INPUT_H_
#include <windows.h>
#include <dinput.h>
#include "KeyDefine.h"

namespace AquaGameSystem {

	/// L[foCX
	enum eKeyDeviceType
	{
		KEY_DEVICE_TYPE_NONE = 0,
		KEY_DEVICE_TYPE_KEYBOARD,
		KEY_DEVICE_TYPE_GAMEPAD,
	};

	/// L[ATC
	enum eKeyAssignmentType
	{
		KEY_ASSIGNMENT_TYPE_START = 0,
		KEY_ASSIGNMENT_TYPE_UP = KEY_ASSIGNMENT_TYPE_START,
		KEY_ASSIGNMENT_TYPE_DOWN,
		KEY_ASSIGNMENT_TYPE_LEFT,
		KEY_ASSIGNMENT_TYPE_RIGHT,
		KEY_ASSIGNMENT_TYPE_BUTTON1,
		KEY_ASSIGNMENT_TYPE_BUTTON2,
		KEY_ASSIGNMENT_TYPE_BUTTON3,
		KEY_ASSIGNMENT_TYPE_BUTTON4,
		KEY_ASSIGNMENT_TYPE_BUTTON5,
		KEY_ASSIGNMENT_TYPE_BUTTON6,
		KEY_ASSIGNMENT_TYPE_BUTTON7,
		KEY_ASSIGNMENT_TYPE_BUTTON8,
		KEY_ASSIGNMENT_TYPE_BUTTON9,
		KEY_ASSIGNMENT_TYPE_BUTTON10,
		KEY_ASSIGNMENT_TYPE_MAX,
	};

	//======================================
	// struct tagMouseCursorPoint
	//======================================
	/** @brief }EXJ[\|Cg\
	 * @note <br>
	 */
	struct tagMouseCursorPoint
	{
		LONG	lX;		/// }EXW
		LONG	lY;		/// }EXWc
		LONG	lZ;		/// }EXWzC[
	};

	//======================================
	// struct tagKeyDevice
	//======================================
	/** @brief L[foCX\
	 * @note <br>
	 */
	struct tagKeyDevice
	{
		eKeyDeviceType	m_type;		/// L[foCX
		int				m_nNumber;	/// L[foCXԍ
	};

	//======================================
	// class CKeyAssignment
	//======================================
	/** @brief L[ATCNX
	 * @note <br>
	 */
	class CKeyAssignment
	{
	private:
		tagKeyDevice	m_KeyDevice;	/// L[foCX
		DWORD			m_KeyID;		/// L[ID

	public:
		/// ftHgRXgN^
		CKeyAssignment();
		/// fXgN^
		virtual ~CKeyAssignment();

		/// L[foCX̎擾
		tagKeyDevice	GetKeyDevice(void) const;
		/// L[foCX̐ݒ
		void			SetKeyDevice(const tagKeyDevice KeyDevice);
		/// L[ID̎擾
		DWORD			GetKeyID(void) const;
		/// L[ID̐ݒ
		void			SetKeyID(DWORD KeyID);
	};

	//======================================
	// class CKeyAssignments
	//======================================
	/** @brief L[ATCNXiL[ATCʑΉpj
	 * @note <br>
	 */
	class CKeyAssignments
	{
	private:
		/// L[ATC
		CKeyAssignment	m_KeyAssignment[KEY_ASSIGNMENT_TYPE_MAX];

	public:
		/// ftHgRXgN^
		CKeyAssignments();
		/// fXgN^
		virtual ~CKeyAssignments();

		/// L[ATC擾
		CKeyAssignment*			GetKeyAssignment(eKeyAssignmentType KeyAssignmentType);
		/// L[ATC擾iconstj
		const CKeyAssignment*	GetKeyAssignment(eKeyAssignmentType KeyAssignmentType) const;
	};

	//======================================
	// class CInputKeyAssignment
	//======================================
	/** @brief ̓L[ATCNX
	 * @note <br>
	 */
	class CInputKeyAssignment
	{
	private:
		/// VXeGXP[vL[ATC
		CKeyAssignment		m_KeyAssignmentSysEsc;
		/// L[ATC
		CKeyAssignments*	m_KeyAssignmentsArray;
		int					m_nKeyAssignmentsArraySize;

	public:
		/// ftHgRXgN^
		CInputKeyAssignment();
		/// RXgN^
		CInputKeyAssignment(int nKeyAssignmentMax);

		/// fXgN^
		virtual ~CInputKeyAssignment();

		/// L[ATCǂݍ
		int							LoadKeyAssignment(LPCSTR pszFileName);
		/// VXeGXP[vL[ATC擾
		CKeyAssignment*				GetKeyAssignmentSysEsc(void);
		/// VXeGXP[vL[ATC擾iconstj
		const CKeyAssignment*		GetKeyAssignmentSysEsc(void) const;
		/// L[ATC擾
		CKeyAssignment*				GetKeyAssignment(int iGameCtrliNum, eKeyAssignmentType KeyAssignmentType);
		/// L[ATC擾iconstj
		const CKeyAssignment*		GetKeyAssignment(int iGameCtrliNum, eKeyAssignmentType KeyAssignmentType) const;

	};


	//==============================================
	// class CInputAbstract
	//==============================================
	/** @brief [U[C^[tFCXNX(ۃNX)
	 * @note GNX|[gpۃNX<br>
	 */
	class CInputAbstract
	{
	public:
		virtual int Init(HINSTANCE hInstance, HWND hWnd, int nDIDevGameCtrlMax = 4) = 0;
		virtual void Term() = 0;
		virtual int LoadKeyAssignment(LPCSTR pszFileName) = 0;
		virtual void UpDateState(HWND hWnd) = 0;
		virtual int GetMouseCursorPos(tagMouseCursorPoint* pPoint) = 0;
		virtual BOOL IsMouseButtons(DWORD dwKey) = 0;
		virtual BOOL GameCtrlCheck(long key, short dev_id) = 0;
		virtual BOOL GameCtrlTrgCheck(long key, short dev_id) = 0;
		virtual BOOL GameCtrlUpTrgCheck(long key, short dev_id) = 0;

	};

	//======================================
	// class CInput
	//======================================
	/** @brief [U[C^[tFCXNX
	 * @note [U[C^[tFCXpfoCX̊ǗA͍XVA͊mFsƂo<br>
	 */
	class CInput : public CInputAbstract
	{
	private:
		static CInput*			st_Instance;						/// CX^X
		DIMOUSESTATE2			m_dims;								/// DirectInput}EX
		tagMouseCursorPoint		m_MouseCursorPoint;					/// }EXJ[\|Cg
		tagMouseCursorPoint		m_DefaultCursorPoint;				/// ftHg}EXJ[\|Cg
		BOOL					m_bUpDateStateMouseFirst;			/// }EXXVtO

		LPDIRECTINPUT8			m_pDInput;							/// DirectInputIuWFNg
		LPDIRECTINPUTDEVICE8	m_pDIDevKeyboard;					/// ftHgL[{[hfoCXIuWFNg
		LPDIRECTINPUTDEVICE8	m_pDIDevMouse;						/// ftHg}EXfoCXIuWFNg
		LPDIRECTINPUTDEVICE8*	m_pDIDevGameCtrl;					/// Q[Rg[foCXIuWFNg
		int						m_nDIDevGameCtrlMax;				/// Q[Rg[foCXIuWFNg̍ő吔
		int						m_nDIDevGameCtrl;					/// Q[Rg[foCXIuWFNg̐

		DWORD*					m_GameCtrlData;						/// Q[Rg[f[^
		DWORD*					m_GameCtrlTrgData;					/// Q[Rg[gKf[^
		DWORD*					m_GameCtrlUpTrgData;				/// Q[Rg[UPgKf[^
		CInputKeyAssignment*	m_InputKeyAssignment;				/// L[ATCNX


		/// L[{[h
		int							InitKeyboard(HWND hWnd);
		/// L[{[h̓͂Jn
		void						AcquireKeyboard(void);
		/// L[{[h̓͂𒆎~
		void						UnacquireKeyboard(void);

		/// }EX
		int							InitMouse(HWND hWnd);
		/// }EX̓͂Jn
		void						AcquireMouse(void);
		/// }EX̓͂𒆎~
		void						UnacquireMouse(void);

		/// Q[Rg[
		int							InitGameCtrl(HWND hWnd);
		/// L[ATCNX擾
		CInputKeyAssignment*		GetInputKeyAssignment(void);
		/// L[ATCNX擾iconstj
		const CInputKeyAssignment*	GetInputKeyAssignment(void) const;

		/// Q[Rg[foCX񋓃R[obN
		static BOOL					EnumGameCtrlCallback(LPCDIDEVICEINSTANCE lpddi, LPVOID pvRef);
		/// Q[Rg[foCX쐬
		BOOL						CreateDeviceGameCtrl(LPCDIDEVICEINSTANCE lpddi);

		/// }EX̏ԂXV
		void						UpDateStateMouse(HWND hWnd);
		/// L[̏ԂXV
		void						UpDateStateKey(void);
		/// L[{[h̏ԂXV
		void						UpDateStateGameCtrl_KeyBoard(int iGameCtrl, eKeyAssignmentType KeyAssignmentType, const CKeyAssignment* pKeyData);
		/// Q[Rg[̏ԂXV
		void						UpDateStateGameCtrl_GamePad(int iGameCtrl, eKeyAssignmentType KeyAssignmentType, const CKeyAssignment* pKeyData);


	public:
		/// ftHgRXgN^
		CInput();
		/// fXgN^
		virtual ~CInput();

		/// CX^X̎擾
		static CInput* GetInstance(void);

		/// 
		virtual int Init(HINSTANCE hInstance, HWND hWnd, int nDIDevGameCtrlMax = 4);
		/// I
		virtual void Term();

		/// L[ATCǂݍ
		virtual int LoadKeyAssignment(LPCSTR pszFileName);

		/// e̓foCX̏ԂXV
		virtual void UpDateState(HWND hWnd);

		/// }EXW擾
		virtual int GetMouseCursorPos(tagMouseCursorPoint* pPoint);
		/// }EXNbNmF
		virtual BOOL IsMouseButtons(DWORD dwKey);
		
		/// Q[Rg[͂mF
		virtual BOOL GameCtrlCheck(long key, short dev_id);
		/// Q[Rg[͂mFigKpj
		virtual BOOL GameCtrlTrgCheck(long key, short dev_id);
		/// Q[Rg[͂mFiUPgKpj
		virtual BOOL GameCtrlUpTrgCheck(long key, short dev_id);

	};
}

#endif // _INPUT_H_


